@@ -12,8 +12,15 @@ RequestSpotInstances API. The --duration-hours, --cores, and
 API.
 
 The return value (stdout) is a JSON object with one key, ``instance_id``.
-"""
 
+Examples:
+
+- Launch an on-demand t3.micro instance with the root volume enlarged to 64GB and another 64GB volume attached at /mnt:
+    aegea launch my-instance --storage /=64GB /mnt=64GB
+
+- Launch a spot r5d.xlarge instance with the home directory mounted on EFS:
+    aegea launch my-instance --spot-price 1 --instance-type r5d.xlarge --efs-home
+"""
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import os, sys, time, datetime, base64, json
@@ -23,6 +30,7 @@ import yaml
 
 from . import register_parser, logger, config
 from .efs import create as create_efs, parser_create as parser_create_efs, __name__ as efs_security_group_name
+from .ssh import get_user_info
 from .util import wait_for_port, validate_hostname, paginate
 from .util.cloudinit import get_user_data
 from .util.aws import (ensure_vpc, ensure_subnet, ensure_security_group, ensure_log_group,
@@ -40,13 +48,12 @@ def get_spot_bid_price(instance_type, ondemand_multiplier=1.2):
     ondemand_price = get_ondemand_price_usd(clients.ec2.meta.region_name, instance_type)
     return float(ondemand_price) * ondemand_multiplier
 
-def get_startup_commands(args, username):
+def get_startup_commands(args):
     hostname = ".".join([args.hostname, config.dns.private_zone.rstrip(".")]) if args.use_dns else args.hostname
     return [
         "hostnamectl set-hostname " + hostname,
         "service awslogs restart",
         "echo tsc > /sys/devices/system/clocksource/clocksource0/current_clocksource",
-        "ssh -oUserKnownHostsFile=/dev/null -oStrictHostKeyChecking=no {}@localhost -N || true".format(username)
     ] + args.commands
 
 def get_ssh_ca_keys(bless_config):
@@ -61,6 +68,11 @@ def get_ssh_ca_keys(bless_config):
     ca_keys = json.loads(ca_keys_secret["SecretString"])["ssh_ca_keys"]
     return "\n".join(ca_keys)
 
+def infer_architecture(instance_type):
+    if any((instance_type.startswith(f) for f in ("x2g", "m6g", "c6g", "r6g", "t4g", "a1"))):
+        return "arm64"
+    return "x86_64"
+
 def launch(args):
     if args.spot_price or args.duration_hours or args.cores or args.min_mem_per_core_gb:
         args.spot = True
@@ -68,6 +80,7 @@ def launch(args):
         dns_zone = DNSZone()
     ssh_key_name = ensure_ssh_key(name=args.ssh_key_name, base_name=__name__,
                                   verify_pem_file=args.verify_ssh_key_pem_file)
+    user_info = get_user_info()
     # TODO: move all account init checks into init helper with region-specific semaphore on s3
     try:
         ensure_log_group("syslog")
@@ -81,16 +94,21 @@ def launch(args):
         validate_hostname(args.hostname)
         assert not args.hostname.startswith("i-")
     ami_tags = dict(tag.split("=", 1) for tag in args.ami_tags or [])
+    arch = infer_architecture(instance_type=args.instance_type)
     if args.ubuntu_linux_ami:
-        args.ami = locate_ami(product="com.ubuntu.cloud:server:20.04:amd64")
+        args.ami = locate_ami("Ubuntu", release="20.04", architecture=arch)
     elif args.amazon_linux_ami:
-        args.ami = get_ssm_parameter("/aws/service/ami-amazon-linux-latest/amzn2-ami-hvm-x86_64-gp2")
+        args.ami = locate_ami("Amazon Linux", release="2", architecture=arch)
     else:
         try:
-            args.ami = resolve_ami(args.ami, **ami_tags)
+            if not (args.ami or args.ami_tags or args.ami_tag_keys):
+                args.ami_tag_keys.append("AegeaVersion")
+            args.ami = resolve_ami(args.ami, tags=ami_tags, tag_keys=args.ami_tag_keys, arch=arch)
+            logger.info("Using %s", args.ami)
         except AegeaException as e:
             if args.ami is None and len(ami_tags) == 0 and "Could not resolve AMI" in str(e):
-                raise AegeaException("No AMI was given, and no AMIs were found in this account. "
+                raise AegeaException("No AMI was given, and no " + arch + " AMIs were found in this account. "
+                                     "To build an aegea AMI, use aegea build-ami --architecture " + arch + ". "
                                      "To use the default Ubuntu Linux LTS AMI, use --ubuntu-linux-ami. "
                                      "To use the default Amazon Linux 2 AMI, use --amazon-linux-ami. ")
             raise
@@ -112,22 +130,27 @@ def launch(args):
                 break
         else:
             create_efs_args = ["aegea_home", "--tags", "mountpoint=/home", "managedBy=aegea"]
-            create_efs(parser_create_efs.parse_args(create_efs_args))
+            create_efs(parser_create_efs.parse_args(create_efs_args), vpc=vpc)
         security_groups.append(resolve_security_group(efs_security_group_name, vpc))
 
     ssh_host_key = new_ssh_key()
     user_data_args = dict(host_key=ssh_host_key,
-                          commands=get_startup_commands(args, ARN.get_iam_username()),
+                          commands=get_startup_commands(args),
                           packages=args.packages,
                           storage=args.storage)
-    if args.bless_config:
+    if args.provision_user:
+        user_data_args["provision_users"] = [dict(name=user_info["linux_username"],
+                                                  uid=user_info["linux_user_id"],
+                                                  sudo="ALL=(ALL) NOPASSWD:ALL",
+                                                  shell="/bin/bash")]
+    elif args.bless_config:
         with open(args.bless_config) as fh:
             bless_config = yaml.safe_load(fh)
         user_data_args["ssh_ca_keys"] = get_ssh_ca_keys(bless_config)
         user_data_args["provision_users"] = bless_config["client_config"]["remote_users"]
 
     hkl = hostkey_line(hostnames=[], key=ssh_host_key).strip()
-    instance_tags = dict(Name=args.hostname, Owner=ARN.get_iam_username(),
+    instance_tags = dict(Name=args.hostname, Owner=user_info["iam_username"],
                          SSHHostPublicKeyPart1=hkl[:255], SSHHostPublicKeyPart2=hkl[255:],
                          OwnerSSHKeyName=ssh_key_name, **dict(args.tags))
     user_data_args.update(dict(args.cloud_config_data))
@@ -135,7 +158,7 @@ def launch(args):
                        KeyName=ssh_key_name,
                        SecurityGroupIds=[sg.id for sg in security_groups],
                        InstanceType=args.instance_type,
-                       BlockDeviceMappings=get_bdm(ebs_storage=args.storage),
+                       BlockDeviceMappings=get_bdm(ami=args.ami, ebs_storage=args.storage),
                        UserData=get_user_data(**user_data_args))
     tag_spec = dict(ResourceType="instance", Tags=encode_tags(instance_tags))
     logger.info("Launch spec user data is %i bytes long", len(launch_spec["UserData"]))
@@ -148,7 +171,7 @@ def launch(args):
     if args.availability_zone:
         launch_spec["Placement"] = dict(AvailabilityZone=args.availability_zone)
     if args.client_token is None:
-        args.client_token = get_client_token(ARN.get_iam_username(), __name__)
+        args.client_token = get_client_token(user_info["iam_username"], __name__)
     try:
         if args.spot:
             launch_spec["UserData"] = base64.b64encode(launch_spec["UserData"]).decode()
@@ -220,14 +243,22 @@ def launch(args):
 
 parser = register_parser(launch)
 parser.add_argument("hostname")
+parser.add_argument("--storage", nargs="+", metavar="MOUNTPOINT=SIZE_GB",
+                    type=lambda x: x.rstrip("GBgb").split("=", 1),
+                    help="At launch time, attach EBS volume(s) of this size, format and mount them.")
+parser.add_argument("--efs-home", action="store_true",
+                    help="Create and manage an EFS filesystem that the instance will use for user home directories")
 parser.add_argument("--commands", nargs="+", metavar="COMMAND", help="Commands to run on host upon startup")
 parser.add_argument("--packages", nargs="+", metavar="PACKAGE", help="APT packages to install on host upon startup")
 parser.add_argument("--ssh-key-name")
 parser.add_argument("--no-verify-ssh-key-pem-file", dest="verify_ssh_key_pem_file", action="store_false")
+parser.add_argument("--no-provision-user", dest="provision_user", action="store_false")
 parser.add_argument("--bless-config", default=os.environ.get("BLESS_CONFIG"),
                     help="Path to a Bless configuration file (or pass via the BLESS_CONFIG environment variable)")
 parser.add_argument("--ami", help="AMI to use for the instance. Default: " + resolve_ami.__doc__)  # type: ignore
 parser.add_argument("--ami-tags", nargs="+", metavar="NAME=VALUE", help="Use the most recent AMI with these tags")
+parser.add_argument("--ami-tag-keys", nargs="+", default=[], metavar="TAG_NAME",
+                    help="Use the most recent AMI with these tag names")
 parser.add_argument("--ubuntu-linux-ami", action="store_true", help="Use the most recent Ubuntu Linux LTS AMI")
 parser.add_argument("--amazon-linux-ami", action="store_true", help="Use the most recent Amazon Linux 2 AMI")
 parser.add_argument("--spot", action="store_true",
@@ -235,7 +266,7 @@ parser.add_argument("--spot", action="store_true",
 parser.add_argument("--duration-hours", type=float, help="Terminate the spot instance after this number of hours")
 parser.add_argument("--cores", type=int, help="Minimum number of cores to request (spot fleet API)")
 parser.add_argument("--min-mem-per-core-gb", type=float)
-parser.add_argument("--instance-type", "-t").completer = instance_type_completer
+parser.add_argument("--instance-type", "-t", help="See https://ec2instances.info/").completer = instance_type_completer
 parser.add_argument("--spot-price", type=float,
                     help="Maximum bid price for spot instances. Defaults to 1.2x the ondemand price.")
 parser.add_argument("--no-dns", dest="use_dns", action="store_false", help="""
@@ -247,14 +278,11 @@ parser.add_argument("--security-groups", nargs="+", metavar="SECURITY_GROUP")
 parser.add_argument("--tags", nargs="+", metavar="NAME=VALUE", type=lambda x: x.split("=", 1),
                     help="Tags to apply to launched instances.")
 parser.add_argument("--wait-for-ssh", action="store_true",
-                    help="Wait for launched instance to begin accepting SSH connections. Security groups and NACLs must permit SSH from launching host.")  # noqa
-parser.add_argument("--efs-home", action="store_true",
-                    help="Create and manage an EFS filesystem that the instance will use for user home directories")
-parser.add_argument("--storage", nargs="+", metavar="MOUNTPOINT=SIZE_GB",
-                    type=lambda x: x.rstrip("GBgb").split("=", 1),
-                    help="At launch time, attach EBS volume(s) of this size, format and mount them.")
+                    help=("Wait for launched instance to begin accepting SSH connections. "
+                          "Security groups and NACLs must permit SSH from launching host."))
 parser.add_argument("--iam-role", default=__name__,
-                    help="Pass this IAM role to the launched instance through an instance profile. Role credentials will become available in the instance metadata.")  # noqa
+                    help=("Pass this IAM role to the launched instance through an instance profile. "
+                          "Role credentials will become available in the instance metadata."))
 parser.add_argument("--iam-policies", nargs="+", metavar="IAM_POLICY_NAME",
                     help="Ensure the default or specified IAM role has the listed IAM managed policies attached")
 parser.add_argument("--cloud-config-data", type=json.loads)
