@@ -17,14 +17,14 @@ YAML file with the format described in
 https://github.com/chanzuckerberg/blessclient/blob/master/examples/config.yml.
 """
 
-import os, sys, argparse, string, datetime, json, base64, time, fnmatch, subprocess
+import os, sys, argparse, string, datetime, json, base64, time, fnmatch, subprocess, hashlib
 
 import boto3, yaml
 
 from . import register_parser, logger
 from .util.aws import resolve_instance_id, resources, clients, ARN
 from .util.crypto import (add_ssh_host_key_to_known_hosts, ensure_local_ssh_key, get_public_key_from_pair,
-                          add_ssh_key_to_agent, get_ssh_key_path)
+                          add_ssh_key_to_agent, get_ssh_key_path, get_ssh_id)
 from .util.printing import BOLD
 from .util.exceptions import AegeaException
 from .util.compat import lru_cache
@@ -75,11 +75,15 @@ def resolve_instance_public_dns(name):
         raise AegeaException(msg.format(instance, getattr(instance, "state", {}).get("Name")))
     return instance.public_dns_name
 
-def get_linux_username():
-    username = ARN.get_iam_username()
-    assert username != "unknown"
-    username, at, domain = username.partition("@")
-    return username
+def get_user_info():
+    res = clients.sts.get_caller_identity()
+    iam_username = ARN(res["Arn"]).resource.split("/")[-1]
+    linux_username, at, domain = iam_username.partition("@")
+    iam_user_id = res["UserId"]
+    iam_user_id, colon, session = iam_user_id.partition(":")
+    user_id_bytes = hashlib.sha256(iam_user_id.encode()).digest()[-2:]
+    linux_user_id = str(2000 + (int.from_bytes(user_id_bytes, byteorder=sys.byteorder) // 2))
+    return dict(iam_username=iam_username, linux_username=linux_username, linux_user_id=linux_user_id)
 
 def get_kms_auth_token(session, bless_config, lambda_regional_config):
     logger.info("Requesting new KMS auth token in %s", lambda_regional_config["aws_region"])
@@ -167,7 +171,7 @@ def match_instance_to_bastion(instance, bastions):
                 return bastion_config
 
 def prepare_ssh_host_opts(username, hostname, bless_config_filename=None, ssh_key_name=__name__, use_kms_auth=True,
-                          use_ssm=True):
+                          use_ssm=True, use_ec2_instance_connect=True):
     if bless_config_filename:
         with open(bless_config_filename) as fh:
             bless_config = yaml.safe_load(fh)
@@ -193,8 +197,17 @@ def prepare_ssh_host_opts(username, hostname, bless_config_filename=None, ssh_ke
         if get_instance(hostname).key_name is not None:
             add_ssh_key_to_agent(get_instance(hostname).key_name)
         if not username:
-            username = get_linux_username()
+            username = get_user_info()["linux_username"]
         save_instance_public_key(hostname, use_ssm=use_ssm)
+        if use_ec2_instance_connect:
+            ssh_public_key = get_ssh_id()
+            logger.info("Sending SSH public key %s for %s to %s", ssh_public_key.split()[-1], username, hostname)
+            clients.ec2_instance_connect.send_ssh_public_key(
+                InstanceId=get_instance(hostname).id,
+                InstanceOSUser=username,
+                SSHPublicKey=ssh_public_key,
+                AvailabilityZone=get_instance(hostname).subnet.availability_zone
+            )
         return [], username + "@" + (get_instance(hostname).id if use_ssm else resolve_instance_public_dns(hostname))
 
 def init_ssm(instance_id):
@@ -213,7 +226,9 @@ def ssh(args):
 
     host_opts, hostname = prepare_ssh_host_opts(username=prefix, hostname=name,
                                                 bless_config_filename=args.bless_config,
-                                                use_kms_auth=args.use_kms_auth, use_ssm=args.use_ssm)
+                                                use_kms_auth=args.use_kms_auth,
+                                                use_ssm=args.use_ssm,
+                                                use_ec2_instance_connect=args.use_ec2_instance_connect)
     os.execvp("ssh", ["ssh"] + ssh_opts + host_opts + [hostname] + args.ssh_args)
 
 ssh_parser = register_parser(ssh, help="Connect to an EC2 instance", description=__doc__)
@@ -223,6 +238,7 @@ ssh_parser.add_argument("ssh_args", nargs=argparse.REMAINDER,
 ssh_parser.add_argument("--server-alive-interval", help=argparse.SUPPRESS)
 ssh_parser.add_argument("--server-alive-count-max", help=argparse.SUPPRESS)
 ssh_parser.add_argument("--no-ssm", action="store_false", dest="use_ssm")
+ssh_parser.add_argument("--no-ec2-instance-connect", action="store_false", dest="use_ec2_instance_connect")
 add_bless_and_passthrough_opts(ssh_parser, "ssh")
 
 def scp(args):
