@@ -5,14 +5,14 @@ from datetime import datetime, timedelta
 from ... import logger
 from .. import constants, VerboseRepr
 from ..exceptions import AegeaException
-from . import ensure_iam_role, clients
+from . import clients
+from .iam import ensure_iam_role
 
 class SpotFleetBuilder(VerboseRepr):
     # TODO: vivify from toolspec; vivify from SFR ID; update with incremental cores/memory requirements
     def __init__(self, launch_spec, cores=1, min_cores_per_instance=1, min_mem_per_core_gb=1.5, gpus_per_instance=0,
-                 min_ephemeral_storage_gb=0, spot_price=None, duration_hours=None, sfr_type="request",
-                 instance_interruption_behavior="terminate", instance_type_prefixes=None,
-                 client_token=None, dry_run=False):
+                 min_ephemeral_storage_gb=0, spot_price=None, duration_hours=None, client_token=None,
+                 instance_type_prefixes=None, dry_run=False):
         if spot_price is None:
             spot_price = 1
         if "SecurityGroupIds" in launch_spec:
@@ -29,10 +29,8 @@ class SpotFleetBuilder(VerboseRepr):
         self.instance_type_prefixes = instance_type_prefixes
         self.dry_run = dry_run
         self.iam_fleet_role = self.get_iam_fleet_role()
-        self.spot_fleet_request_config = dict(Type=sfr_type,
-                                              SpotPrice=str(spot_price),
+        self.spot_fleet_request_config = dict(SpotPrice=str(spot_price),
                                               TargetCapacity=cores,
-                                              InstanceInterruptionBehavior=instance_interruption_behavior,
                                               IamFleetRole=self.iam_fleet_role.arn)
         if client_token:
             self.spot_fleet_request_config.update(ClientToken=client_token)
